@@ -33,6 +33,10 @@
 #define setmaxT ((4096-12)/sizeof(TObject))
 #define setmaxTinc ((4096)/sizeof(TObject))
 
+#define RED_CANONICALIZE 200
+#define REDNF_CANONICALIZE 60
+#define REDTAIL_CANONICALIZE 100
+
 // if you want std computations as in Singular version < 2:
 // This disables RedThrough, tailReductions against T (bba),
 // sets posInT = posInT15 (bba, strat->honey), and enables redFirst with LDeg
@@ -167,6 +171,7 @@ public:
   // manipulations
   KINLINE void pNorm();
   KINLINE void pCleardenom();
+  KINLINE void pContent();
 
 #ifdef KDEBUG
   void wrp();
@@ -261,7 +266,6 @@ public:
   KINLINE void HeadNormalize();
 };
 
-
 EXTERN_VAR int HCord;
 
 class skStrategy
@@ -322,11 +326,9 @@ public:
   TSet T;
   LSet L;
   LSet    B;
-  poly    kHEdge;
   poly    kNoether;
-  poly    t_kHEdge; // same polys in tailring
+  poly    t_kNoether; // same polys in tailring
   KINLINE poly    kNoetherTail();
-  poly    t_kNoether;
   BOOLEAN * NotUsedAxis;
   BOOLEAN * pairtest;/*used for enterOnePair*/
   poly tail;
@@ -350,7 +352,6 @@ public:
   int Bl,Bmax;
   int ak,LazyDegree,LazyPass;
   int syzComp;
-  int HCord;
   int lastAxis;
   int newIdeal;
   int minim;
@@ -372,7 +373,7 @@ public:
 #ifdef HAVE_PLURAL
   /*BOOLEAN*/ char z2homog; // Z_2 - homogeneous input allows product criterion in commutative and SCA cases!
 #endif
-  /*BOOLEAN*/ char kHEdgeFound;
+  /*BOOLEAN*/ char kAllAxis; // all axis are used -> (re)compute noether
   /*BOOLEAN*/ char honey,sugarCrit;
   /*BOOLEAN*/ char Gebauer,noTailReduction;
   /*BOOLEAN*/ char fromT;
@@ -490,7 +491,9 @@ int posInL110 (const LSet set, const int length,
 KINLINE poly redtailBba (poly p,int end_pos,kStrategy strat,BOOLEAN normalize=FALSE);
 KINLINE poly redtailBbaBound (poly p,int end_pos,kStrategy strat,int bound,BOOLEAN normalize=FALSE);
 #ifdef HAVE_RINGS
+KINLINE poly redtailBba_Ring (poly p,int end_pos,kStrategy strat);
 KINLINE poly redtailBba_Z (poly p,int end_pos,kStrategy strat);
+poly redtailBba_Ring (LObject* L, int end_pos, kStrategy strat );
 poly redtailBba_Z (LObject* L, int end_pos, kStrategy strat );
 void redtailBbaAlsoLC_Z (LObject* L, int end_pos, kStrategy strat );
 #endif
@@ -507,6 +510,8 @@ poly redNF (poly h,int & max_ind,int nonorm,kStrategy strat);
 int redNF0 (LObject *P,kStrategy strat);
 poly redNFTail (poly h,const int sl,kStrategy strat);
 int redHoney (LObject* h, kStrategy strat);
+int redHoneyM (LObject* h, kStrategy strat);
+int redLiftstd (LObject* h, kStrategy strat);
 #ifdef HAVE_RINGS
 int redRing (LObject* h,kStrategy strat);
 int redRing_Z (LObject* h,kStrategy strat);
@@ -515,12 +520,6 @@ void enterExtendedSpoly(poly h,kStrategy strat);
 void enterExtendedSpolySig(poly h,poly hSig,kStrategy strat);
 void superenterpairs (poly h,int k,int ecart,int pos,kStrategy strat, int atR = -1);
 void superenterpairsSig (poly h,poly hSig,int hFrom,int k,int ecart,int pos,kStrategy strat, int atR = -1);
-poly kCreateZeroPoly(long exp[], long cabsind, poly* t_p, ring leadRing, ring tailRing);
-long ind2(long arg);
-
-long ind_fact_2(long arg);
-long twoPow(long arg);
-ideal createG0();
 #endif
 int redLazy (LObject* h,kStrategy strat);
 int redHomog (LObject* h,kStrategy strat);
@@ -594,6 +593,9 @@ BOOLEAN findMinLMPair(poly sig, unsigned long not_sevSig, kStrategy strat, int s
 
 /// returns index of p in TSet, or -1 if not found
 int kFindInT(poly p, TSet T, int tlength);
+#ifdef HAVE_SHIFTBBA
+int kFindInTShift(poly p, TSet T, int tlength);
+#endif
 
 /// return -1 if no divisor is found
 ///        number of first divisor in T, otherwise
@@ -645,17 +647,17 @@ BOOLEAN kTest(kStrategy strat);
 // test strat, and test that S is contained in T
 BOOLEAN kTest_TS(kStrategy strat);
 // test LObject
-BOOLEAN kTest_L(LObject* L, ring tailRing,
+BOOLEAN kTest_L(LObject* L, kStrategy strat,
                  BOOLEAN testp = FALSE, int lpos = -1,
                  TSet T = NULL, int tlength = -1);
 // test TObject
-BOOLEAN kTest_T(TObject* T, ring tailRing = NULL, int tpos = -1, char TN = '?');
+BOOLEAN kTest_T(TObject* T, kStrategy strat, int tpos = -1, char TN = '?');
 // test set strat->SevS
 BOOLEAN kTest_S(kStrategy strat);
 #else
 #define kTest(A)        (TRUE)
 #define kTest_TS(A)     (TRUE)
-#define kTest_T(T)      (TRUE)
+#define kTest_T(T,S)    (TRUE)
 #define kTest_S(T)      (TRUE)
 #define kTest_L(T,R)    (TRUE)
 #endif
@@ -701,6 +703,7 @@ int ksReducePoly(LObject* PR,
                  TObject* PW,
                  poly spNoether = NULL,
                  number *coef = NULL,
+                 poly *mon =NULL,
                  kStrategy strat = NULL);
 
 /* like ksReducePoly, but if the reducer has only 1 term we still
@@ -844,6 +847,16 @@ KINLINE void clearS (poly p, unsigned long p_sev, int* at, int* k,
 /* shiftgb stuff */
 #include "kernel/GBEngine/shiftgb.h"
 
+#ifdef HAVE_SHIFTBBA
+static inline int kFindInL1(const poly p, const kStrategy strat)
+{
+  for(int i=strat->Ll;i>=0;i--)
+  {
+    if (p==strat->L[i].p1) return i;
+  }
+  return -1;
+}
+
 poly pMove2CurrTail(poly p, kStrategy strat);
 
 poly pMoveCurrTail2poly(poly p, kStrategy strat);
@@ -852,17 +865,19 @@ poly pCopyL2p(LObject h, kStrategy strat);
 
 void enterTShift(LObject p, kStrategy strat, int atT = -1);
 
-void enterOnePairShift (poly q, poly p, int ecart, int isFromQ, kStrategy strat, int atR, int ecartq, int qisFromQ, int shiftcount, int ifromS);
+BOOLEAN enterOnePairShift (poly q, poly p, int ecart, int isFromQ, kStrategy strat, int atR, int ecartq, int qisFromQ, int shiftcount, int ifromS);
 
 void enterpairsShift (poly h,int k,int ecart,int pos,kStrategy strat, int atR);
 
-void initBbaShift(kStrategy strat);
+void superenterpairsShift (poly h,int k,int ecart,int pos,kStrategy strat, int atR);
 
 poly redtailBbaShift (LObject* L, int pos, kStrategy strat, BOOLEAN withT, BOOLEAN normalize);
 
 int redFirstShift (LObject* h,kStrategy strat); // ok
 
 ideal bbaShift(ideal F, ideal Q,intvec *w,intvec *hilb,kStrategy strat);
+#endif
+
 // test syz strategy: // will be removed soon
 EXTERN_VAR int (*test_PosInT)(const TSet T,const int tl,LObject &h);
 EXTERN_VAR int (*test_PosInL)(const LSet set, const int length,
@@ -881,4 +896,6 @@ static inline void kDeleteLcm(LObject *P)
    P->lcm=NULL;
  }
 }
+
+void initenterpairs (poly h,int k,int ecart,int isFromQ,kStrategy strat, int atR = -1);
 #endif
