@@ -284,7 +284,7 @@ struct ip_sring
 
   omBin    PolyBin; /* Bin from where monoms are allocated */
   intvec * pModW;   /* std: module weights */
-  poly      ppNoether; /*  variables, set by procedures from hecke/kstd1:
+  poly     ppNoether; /*  variables, set by procedures from hecke/kstd1:
                             the highest monomial below pHEdge */
   void * ext_ref;   /* libsing GAP object */
 // #ifdef HAVE_RINGS
@@ -311,6 +311,7 @@ struct ip_sring
 
 #ifdef HAVE_SHIFTBBA
   short          isLPring; /* 0 for non-letterplace rings, otherwise the number of LP blocks, at least 1, known also as lV */
+  short          LPncGenCount;
 #endif
 
   BOOLEAN   VectorOut;
@@ -345,8 +346,10 @@ struct ip_sring
      VarL_Offset[i] gets i-th long var in exp vector */
   int*      VarL_Offset;
 
-  /* mask for getting single exponents */
+  /* mask for getting single exponents, also maxExp */
   unsigned long bitmask;
+  /* wanted maxExp */
+  unsigned long wanted_maxExp;
   /* mask used for divisiblity tests */
   unsigned long divmask; // rComplete
 
@@ -417,7 +420,8 @@ static inline BOOLEAN rIsLPRing(const ring r)
 
 static inline BOOLEAN rIsNCRing(const ring r)
 {
-  return rIsLPRing(r) || rIsPluralRing(r);
+  assume(r != NULL);
+  return rIsPluralRing(r) || rIsLPRing(r);
 }
 
 static inline BOOLEAN rIsRatGRing(const ring r)
@@ -466,8 +470,11 @@ BOOLEAN rSamePolyRep(ring r1, ring r2);
 
 void   rUnComplete(ring r);
 
-BOOLEAN rRing_is_Homog(ring r);
-BOOLEAN rRing_has_CompLastBlock(ring r);
+BOOLEAN rRing_is_Homog(const ring r);
+BOOLEAN rRing_has_CompLastBlock(const ring r);
+BOOLEAN rRing_ord_pure_dp(const ring r);
+BOOLEAN rRing_ord_pure_Dp(const ring r);
+BOOLEAN rRing_ord_pure_lp(const ring r);
 
 #ifdef HAVE_RINGS
 static inline BOOLEAN rField_is_Ring_2toM(const ring r)
@@ -476,8 +483,7 @@ static inline BOOLEAN rField_is_Ring_2toM(const ring r)
 static inline BOOLEAN rField_is_Ring_PtoM(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return ( nCoeff_is_Ring_PtoM(r->cf) ); }
 
-static inline BOOLEAN rField_is_Ring(const ring r)
-{ assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Ring(r->cf); }
+#define rField_is_Ring(R) nCoeff_is_Ring((R)->cf)
 
 static inline BOOLEAN rField_is_Domain(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Domain(r->cf); }
@@ -560,7 +566,7 @@ BOOLEAN rComplete(ring r, int force = 0);
 /// set all properties of a new ring - also called by rComplete
 void p_SetGlobals(const ring r, BOOLEAN complete = TRUE);
 
-static inline int rBlocks(ring r)
+static inline int rBlocks(const ring r)
 {
   assume(r != NULL);
   int i=0;
@@ -833,4 +839,7 @@ ring rPlusVar(const ring r, char *v,int left);
 
 /// undo rPlusVar
 ring rMinusVar(const ring r, char *v);
+
+static inline ring rIncRefCnt(ring r) { r->ref++; return r; }
+static inline void rDecRefCnt(ring r) { r->ref--; }
 #endif
